]> git.llucax.com Git - software/libev.git/blobdiff - ev.c
*** empty log message ***
[software/libev.git] / ev.c
diff --git a/ev.c b/ev.c
index 14046a2b3b1206a3460f824d14fd7c67880e5373..3e008a4bce0b0d60d9f9938ccec2f6af9f881b45 100644 (file)
--- a/ev.c
+++ b/ev.c
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #ifndef EV_STANDALONE
 # include "config.h"
 
 # if HAVE_CLOCK_GETTIME
 #ifndef EV_STANDALONE
 # include "config.h"
 
 # if HAVE_CLOCK_GETTIME
-#  define EV_USE_MONOTONIC 1
-#  define EV_USE_REALTIME  1
+#  ifndef EV_USE_MONOTONIC
+#   define EV_USE_MONOTONIC 1
+#  endif
+#  ifndef EV_USE_REALTIME
+#   define EV_USE_REALTIME  1
+#  endif
 # endif
 
 # endif
 
-# if HAVE_SELECT && HAVE_SYS_SELECT_H
+# if HAVE_SELECT && HAVE_SYS_SELECT_H && !defined (EV_USE_SELECT)
 #  define EV_USE_SELECT 1
 # endif
 
 #  define EV_USE_SELECT 1
 # endif
 
-# if HAVE_POLL && HAVE_POLL_H
+# if HAVE_POLL && HAVE_POLL_H && !defined (EV_USE_POLL)
 #  define EV_USE_POLL 1
 # endif
 
 #  define EV_USE_POLL 1
 # endif
 
-# if HAVE_EPOLL && HAVE_EPOLL_CTL && HAVE_SYS_EPOLL_H
+# if HAVE_EPOLL && HAVE_EPOLL_CTL && HAVE_SYS_EPOLL_H && !defined (EV_USE_EPOLL)
 #  define EV_USE_EPOLL 1
 # endif
 
 #  define EV_USE_EPOLL 1
 # endif
 
-# if HAVE_KQUEUE && HAVE_WORKING_KQUEUE && HAVE_SYS_EVENT_H && HAVE_SYS_QUEUE_H
+# if HAVE_KQUEUE && HAVE_SYS_EVENT_H && HAVE_SYS_QUEUE_H && !defined (EV_USE_KQUEUE)
 #  define EV_USE_KQUEUE 1
 # endif
 
 #  define EV_USE_KQUEUE 1
 # endif
 
@@ -154,7 +163,9 @@ typedef struct ev_watcher_time *WT;
 
 static int have_monotonic; /* did clock_gettime (CLOCK_MONOTONIC) work? */
 
 
 static int have_monotonic; /* did clock_gettime (CLOCK_MONOTONIC) work? */
 
-#include "ev_win32.c"
+#ifdef WIN32
+# include "ev_win32.c"
+#endif
 
 /*****************************************************************************/
 
 
 /*****************************************************************************/
 
@@ -223,6 +234,7 @@ typedef struct
 
   struct ev_loop
   {
 
   struct ev_loop
   {
+    ev_tstamp ev_rt_now;
     #define VAR(name,decl) decl;
       #include "ev_vars.h"
     #undef VAR
     #define VAR(name,decl) decl;
       #include "ev_vars.h"
     #undef VAR
@@ -234,6 +246,7 @@ typedef struct
 
 #else
 
 
 #else
 
+  ev_tstamp ev_rt_now;
   #define VAR(name,decl) static decl;
     #include "ev_vars.h"
   #undef VAR
   #define VAR(name,decl) static decl;
     #include "ev_vars.h"
   #undef VAR
@@ -244,7 +257,7 @@ typedef struct
 
 /*****************************************************************************/
 
 
 /*****************************************************************************/
 
-inline ev_tstamp
+ev_tstamp
 ev_time (void)
 {
 #if EV_USE_REALTIME
 ev_time (void)
 {
 #if EV_USE_REALTIME
@@ -273,11 +286,13 @@ get_clock (void)
   return ev_time ();
 }
 
   return ev_time ();
 }
 
+#if EV_MULTIPLICITY
 ev_tstamp
 ev_now (EV_P)
 {
 ev_tstamp
 ev_now (EV_P)
 {
-  return rt_now;
+  return ev_rt_now;
 }
 }
+#endif
 
 #define array_roundsize(type,n) ((n) | 4 & ~3)
 
 
 #define array_roundsize(type,n) ((n) | 4 & ~3)
 
@@ -737,10 +752,10 @@ loop_init (EV_P_ int methods)
       }
 #endif
 
       }
 #endif
 
-      rt_now    = ev_time ();
+      ev_rt_now = ev_time ();
       mn_now    = get_clock ();
       now_floor = mn_now;
       mn_now    = get_clock ();
       now_floor = mn_now;
-      rtmn_diff = rt_now - mn_now;
+      rtmn_diff = ev_rt_now - mn_now;
 
       if (methods == EVMETHOD_AUTO)
         if (!enable_secure () && getenv ("LIBEV_METHODS"))
 
       if (methods == EVMETHOD_AUTO)
         if (!enable_secure () && getenv ("LIBEV_METHODS"))
@@ -797,7 +812,9 @@ loop_destroy (EV_P)
   /* have to use the microsoft-never-gets-it-right macro */
   array_free_microshit (fdchange);
   array_free_microshit (timer);
   /* have to use the microsoft-never-gets-it-right macro */
   array_free_microshit (fdchange);
   array_free_microshit (timer);
+#if EV_PERIODICS
   array_free_microshit (periodic);
   array_free_microshit (periodic);
+#endif
   array_free_microshit (idle);
   array_free_microshit (prepare);
   array_free_microshit (check);
   array_free_microshit (idle);
   array_free_microshit (prepare);
   array_free_microshit (check);
@@ -980,7 +997,11 @@ timers_reify (EV_P)
       if (w->repeat)
         {
           assert (("negative ev_timer repeat value found while processing timers", w->repeat > 0.));
       if (w->repeat)
         {
           assert (("negative ev_timer repeat value found while processing timers", w->repeat > 0.));
-          ((WT)w)->at = mn_now + w->repeat;
+
+          ((WT)w)->at += w->repeat;
+          if (((WT)w)->at < mn_now)
+            ((WT)w)->at = mn_now;
+
           downheap ((WT *)timers, timercnt, 0);
         }
       else
           downheap ((WT *)timers, timercnt, 0);
         }
       else
@@ -990,10 +1011,11 @@ timers_reify (EV_P)
     }
 }
 
     }
 }
 
+#if EV_PERIODICS
 static void
 periodics_reify (EV_P)
 {
 static void
 periodics_reify (EV_P)
 {
-  while (periodiccnt && ((WT)periodics [0])->at <= rt_now)
+  while (periodiccnt && ((WT)periodics [0])->at <= ev_rt_now)
     {
       struct ev_periodic *w = periodics [0];
 
     {
       struct ev_periodic *w = periodics [0];
 
@@ -1002,15 +1024,15 @@ periodics_reify (EV_P)
       /* first reschedule or stop timer */
       if (w->reschedule_cb)
         {
       /* first reschedule or stop timer */
       if (w->reschedule_cb)
         {
-          ev_tstamp at = ((WT)w)->at = w->reschedule_cb (w, rt_now + 0.0001);
+          ev_tstamp at = ((WT)w)->at = w->reschedule_cb (w, ev_rt_now + 0.0001);
 
 
-          assert (("ev_periodic reschedule callback returned time in the past", ((WT)w)->at > rt_now));
+          assert (("ev_periodic reschedule callback returned time in the past", ((WT)w)->at > ev_rt_now));
           downheap ((WT *)periodics, periodiccnt, 0);
         }
       else if (w->interval)
         {
           downheap ((WT *)periodics, periodiccnt, 0);
         }
       else if (w->interval)
         {
-          ((WT)w)->at += floor ((rt_now - ((WT)w)->at) / w->interval + 1.) * w->interval;
-          assert (("ev_periodic timeout in the past detected while processing timers, negative interval?", ((WT)w)->at > rt_now));
+          ((WT)w)->at += floor ((ev_rt_now - ((WT)w)->at) / w->interval + 1.) * w->interval;
+          assert (("ev_periodic timeout in the past detected while processing timers, negative interval?", ((WT)w)->at > ev_rt_now));
           downheap ((WT *)periodics, periodiccnt, 0);
         }
       else
           downheap ((WT *)periodics, periodiccnt, 0);
         }
       else
@@ -1031,15 +1053,16 @@ periodics_reschedule (EV_P)
       struct ev_periodic *w = periodics [i];
 
       if (w->reschedule_cb)
       struct ev_periodic *w = periodics [i];
 
       if (w->reschedule_cb)
-        ((WT)w)->at = w->reschedule_cb (w, rt_now);
+        ((WT)w)->at = w->reschedule_cb (w, ev_rt_now);
       else if (w->interval)
       else if (w->interval)
-        ((WT)w)->at += ceil ((rt_now - ((WT)w)->at) / w->interval) * w->interval;
+        ((WT)w)->at += ceil ((ev_rt_now - ((WT)w)->at) / w->interval) * w->interval;
     }
 
   /* now rebuild the heap */
   for (i = periodiccnt >> 1; i--; )
     downheap ((WT *)periodics, periodiccnt, i);
 }
     }
 
   /* now rebuild the heap */
   for (i = periodiccnt >> 1; i--; )
     downheap ((WT *)periodics, periodiccnt, i);
 }
+#endif
 
 inline int
 time_update_monotonic (EV_P)
 
 inline int
 time_update_monotonic (EV_P)
@@ -1048,13 +1071,13 @@ time_update_monotonic (EV_P)
 
   if (expect_true (mn_now - now_floor < MIN_TIMEJUMP * .5))
     {
 
   if (expect_true (mn_now - now_floor < MIN_TIMEJUMP * .5))
     {
-      rt_now = rtmn_diff + mn_now;
+      ev_rt_now = rtmn_diff + mn_now;
       return 0;
     }
   else
     {
       now_floor = mn_now;
       return 0;
     }
   else
     {
       now_floor = mn_now;
-      rt_now = ev_time ();
+      ev_rt_now = ev_time ();
       return 1;
     }
 }
       return 1;
     }
 }
@@ -1073,17 +1096,19 @@ time_update (EV_P)
 
           for (i = 4; --i; ) /* loop a few times, before making important decisions */
             {
 
           for (i = 4; --i; ) /* loop a few times, before making important decisions */
             {
-              rtmn_diff = rt_now - mn_now;
+              rtmn_diff = ev_rt_now - mn_now;
 
               if (fabs (odiff - rtmn_diff) < MIN_TIMEJUMP)
                 return; /* all is well */
 
 
               if (fabs (odiff - rtmn_diff) < MIN_TIMEJUMP)
                 return; /* all is well */
 
-              rt_now    = ev_time ();
+              ev_rt_now = ev_time ();
               mn_now    = get_clock ();
               now_floor = mn_now;
             }
 
               mn_now    = get_clock ();
               now_floor = mn_now;
             }
 
+# if EV_PERIODICS
           periodics_reschedule (EV_A);
           periodics_reschedule (EV_A);
+# endif
           /* no timer adjustment, as the monotonic clock doesn't jump */
           /* timers_reschedule (EV_A_ rtmn_diff - odiff) */
         }
           /* no timer adjustment, as the monotonic clock doesn't jump */
           /* timers_reschedule (EV_A_ rtmn_diff - odiff) */
         }
@@ -1091,18 +1116,20 @@ time_update (EV_P)
   else
 #endif
     {
   else
 #endif
     {
-      rt_now = ev_time ();
+      ev_rt_now = ev_time ();
 
 
-      if (expect_false (mn_now > rt_now || mn_now < rt_now - MAX_BLOCKTIME - MIN_TIMEJUMP))
+      if (expect_false (mn_now > ev_rt_now || mn_now < ev_rt_now - MAX_BLOCKTIME - MIN_TIMEJUMP))
         {
         {
+#if EV_PERIODICS
           periodics_reschedule (EV_A);
           periodics_reschedule (EV_A);
+#endif
 
           /* adjust timers. this is easy, as the offset is the same for all */
           for (i = 0; i < timercnt; ++i)
 
           /* adjust timers. this is easy, as the offset is the same for all */
           for (i = 0; i < timercnt; ++i)
-            ((WT)timers [i])->at += rt_now - mn_now;
+            ((WT)timers [i])->at += ev_rt_now - mn_now;
         }
 
         }
 
-      mn_now = rt_now;
+      mn_now = ev_rt_now;
     }
 }
 
     }
 }
 
@@ -1152,8 +1179,8 @@ ev_loop (EV_P_ int flags)
       else
 #endif
         {
       else
 #endif
         {
-          rt_now = ev_time ();
-          mn_now rt_now;
+          ev_rt_now = ev_time ();
+          mn_now    = ev_rt_now;
         }
 
       if (flags & EVLOOP_NONBLOCK || idlecnt)
         }
 
       if (flags & EVLOOP_NONBLOCK || idlecnt)
@@ -1168,23 +1195,27 @@ ev_loop (EV_P_ int flags)
               if (block > to) block = to;
             }
 
               if (block > to) block = to;
             }
 
+#if EV_PERIODICS
           if (periodiccnt)
             {
           if (periodiccnt)
             {
-              ev_tstamp to = ((WT)periodics [0])->at - rt_now + method_fudge;
+              ev_tstamp to = ((WT)periodics [0])->at - ev_rt_now + method_fudge;
               if (block > to) block = to;
             }
               if (block > to) block = to;
             }
+#endif
 
           if (block < 0.) block = 0.;
         }
 
       method_poll (EV_A_ block);
 
 
           if (block < 0.) block = 0.;
         }
 
       method_poll (EV_A_ block);
 
/* update rt_now, do magic */
       time_update (EV_A);
 
       /* queue pending timers and reschedule them */
       timers_reify (EV_A); /* relative timers called last */
       time_update (EV_A);
 
       /* queue pending timers and reschedule them */
       timers_reify (EV_A); /* relative timers called last */
+#if EV_PERIODICS
       periodics_reify (EV_A); /* absolute timers called first */
       periodics_reify (EV_A); /* absolute timers called first */
+#endif
 
       /* queue idle watchers unless io or timers are pending */
       if (idlecnt && !any_pending (EV_A))
 
       /* queue idle watchers unless io or timers are pending */
       if (idlecnt && !any_pending (EV_A))
@@ -1285,6 +1316,8 @@ ev_io_stop (EV_P_ struct ev_io *w)
   if (!ev_is_active (w))
     return;
 
   if (!ev_is_active (w))
     return;
 
+  assert (("ev_io_start called with illegal fd (must stay constant after start!)", w->fd >= 0 && w->fd < anfdmax));
+
   wlist_del ((WL *)&anfds[w->fd].head, (WL)w);
   ev_stop (EV_A_ (W)w);
 
   wlist_del ((WL *)&anfds[w->fd].head, (WL)w);
   ev_stop (EV_A_ (W)w);
 
@@ -1324,7 +1357,7 @@ ev_timer_stop (EV_P_ struct ev_timer *w)
       downheap ((WT *)timers, timercnt, ((W)w)->active - 1);
     }
 
       downheap ((WT *)timers, timercnt, ((W)w)->active - 1);
     }
 
-  ((WT)w)->at = w->repeat;
+  ((WT)w)->at -= mn_now;
 
   ev_stop (EV_A_ (W)w);
 }
 
   ev_stop (EV_A_ (W)w);
 }
@@ -1343,6 +1376,7 @@ ev_timer_again (EV_P_ struct ev_timer *w)
     ev_timer_start (EV_A_ w);
 }
 
     ev_timer_start (EV_A_ w);
 }
 
+#if EV_PERIODICS
 void
 ev_periodic_start (EV_P_ struct ev_periodic *w)
 {
 void
 ev_periodic_start (EV_P_ struct ev_periodic *w)
 {
@@ -1350,12 +1384,12 @@ ev_periodic_start (EV_P_ struct ev_periodic *w)
     return;
 
   if (w->reschedule_cb)
     return;
 
   if (w->reschedule_cb)
-    ((WT)w)->at = w->reschedule_cb (w, rt_now);
+    ((WT)w)->at = w->reschedule_cb (w, ev_rt_now);
   else if (w->interval)
     {
       assert (("ev_periodic_start called with negative interval value", w->interval >= 0.));
       /* this formula differs from the one in periodic_reify because we do not always round up */
   else if (w->interval)
     {
       assert (("ev_periodic_start called with negative interval value", w->interval >= 0.));
       /* this formula differs from the one in periodic_reify because we do not always round up */
-      ((WT)w)->at += ceil ((rt_now - ((WT)w)->at) / w->interval) * w->interval;
+      ((WT)w)->at += ceil ((ev_rt_now - ((WT)w)->at) / w->interval) * w->interval;
     }
 
   ev_start (EV_A_ (W)w, ++periodiccnt);
     }
 
   ev_start (EV_A_ (W)w, ++periodiccnt);
@@ -1391,6 +1425,7 @@ ev_periodic_again (EV_P_ struct ev_periodic *w)
   ev_periodic_stop (EV_A_ w);
   ev_periodic_start (EV_A_ w);
 }
   ev_periodic_stop (EV_A_ w);
   ev_periodic_start (EV_A_ w);
 }
+#endif
 
 void
 ev_idle_start (EV_P_ struct ev_idle *w)
 
 void
 ev_idle_start (EV_P_ struct ev_idle *w)
@@ -1451,7 +1486,7 @@ void
 ev_check_stop (EV_P_ struct ev_check *w)
 {
   ev_clear_pending (EV_A_ (W)w);
 ev_check_stop (EV_P_ struct ev_check *w)
 {
   ev_clear_pending (EV_A_ (W)w);
-  if (ev_is_active (w))
+  if (!ev_is_active (w))
     return;
 
   checks [((W)w)->active - 1] = checks [--checkcnt];
     return;
 
   checks [((W)w)->active - 1] = checks [--checkcnt];
@@ -1522,7 +1557,7 @@ void
 ev_child_stop (EV_P_ struct ev_child *w)
 {
   ev_clear_pending (EV_A_ (W)w);
 ev_child_stop (EV_P_ struct ev_child *w)
 {
   ev_clear_pending (EV_A_ (W)w);
-  if (ev_is_active (w))
+  if (!ev_is_active (w))
     return;
 
   wlist_del ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
     return;
 
   wlist_del ((WL *)&childs [w->pid & (PID_HASHSIZE - 1)], (WL)w);
@@ -1592,3 +1627,7 @@ ev_once (EV_P_ int fd, int events, ev_tstamp timeout, void (*cb)(int revents, vo
     }
 }
 
     }
 }
 
+#ifdef __cplusplus
+}
+#endif
+
@@ -1168,23 +1195,27 @@ ev_loop (EV_P_ int flags)
       method_poll (EV_A_ block);
 
-      /* update rt_now, do magic */
+      /* update ev_rt_now, do magic */
       time_update (EV_A);
 